/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizactivatedetail.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.bizactivatedetail.entity.BizActivateDetail;
import vip.xiaonuo.biz.modular.bizactivatedetail.param.BizActivateDetailAddParam;
import vip.xiaonuo.biz.modular.bizactivatedetail.param.BizActivateDetailEditParam;
import vip.xiaonuo.biz.modular.bizactivatedetail.param.BizActivateDetailIdParam;
import vip.xiaonuo.biz.modular.bizactivatedetail.param.BizActivateDetailPageParam;
import vip.xiaonuo.biz.modular.bizactivatedetail.service.BizActivateDetailService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 激活明细控制器
 *
 * @author fanzherong
 * @date  2025/03/05 14:33
 */
@Tag(name = "激活明细控制器")
@RestController
@Validated
public class BizActivateDetailController {

    @Resource
    private BizActivateDetailService bizActivateDetailService;

    /**
     * 获取激活明细分页
     *
     * @author fanzherong
     * @date  2025/03/05 14:33
     */
    @Operation(summary = "获取激活明细分页")
    @SaCheckPermission("/biz/bizactivatedetail/page")
    @GetMapping("/biz/bizactivatedetail/page")
    public CommonResult<Page<BizActivateDetail>> page(BizActivateDetailPageParam bizActivateDetailPageParam) {
        return CommonResult.data(bizActivateDetailService.page(bizActivateDetailPageParam));
    }

    /**
     * 添加激活明细
     *
     * @author fanzherong
     * @date  2025/03/05 14:33
     */
    @Operation(summary = "添加激活明细")
    @CommonLog("添加激活明细")
    @SaCheckPermission("/biz/bizactivatedetail/add")
    @PostMapping("/biz/bizactivatedetail/add")
    public CommonResult<String> add(@RequestBody @Valid BizActivateDetailAddParam bizActivateDetailAddParam) {
        bizActivateDetailService.add(bizActivateDetailAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑激活明细
     *
     * @author fanzherong
     * @date  2025/03/05 14:33
     */
    @Operation(summary = "编辑激活明细")
    @CommonLog("编辑激活明细")
    @SaCheckPermission("/biz/bizactivatedetail/edit")
    @PostMapping("/biz/bizactivatedetail/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizActivateDetailEditParam bizActivateDetailEditParam) {
        bizActivateDetailService.edit(bizActivateDetailEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除激活明细
     *
     * @author fanzherong
     * @date  2025/03/05 14:33
     */
    @Operation(summary = "删除激活明细")
    @CommonLog("删除激活明细")
    @SaCheckPermission("/biz/bizactivatedetail/delete")
    @PostMapping("/biz/bizactivatedetail/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizActivateDetailIdParam> bizActivateDetailIdParamList) {
        bizActivateDetailService.delete(bizActivateDetailIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取激活明细详情
     *
     * @author fanzherong
     * @date  2025/03/05 14:33
     */
    @Operation(summary = "获取激活明细详情")
    @SaCheckPermission("/biz/bizactivatedetail/detail")
    @GetMapping("/biz/bizactivatedetail/detail")
    public CommonResult<BizActivateDetail> detail(@Valid BizActivateDetailIdParam bizActivateDetailIdParam) {
        return CommonResult.data(bizActivateDetailService.detail(bizActivateDetailIdParam));
    }
}
