/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.disablerecord.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.disablerecord.entity.BizDisableRecord;
import vip.xiaonuo.biz.modular.disablerecord.param.BizDisableRecordAddParam;
import vip.xiaonuo.biz.modular.disablerecord.param.BizDisableRecordEditParam;
import vip.xiaonuo.biz.modular.disablerecord.param.BizDisableRecordIdParam;
import vip.xiaonuo.biz.modular.disablerecord.param.BizDisableRecordPageParam;
import vip.xiaonuo.biz.modular.disablerecord.service.BizDisableRecordService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 清除记录控制器
 *
 * @author fanzherong
 * @date  2025/02/14 11:33
 */
@Tag(name = "清除记录控制器")
@RestController
@Validated
public class BizDisableRecordController {

    @Resource
    private BizDisableRecordService bizDisableRecordService;

    /**
     * 获取清除记录分页
     *
     * @author fanzherong
     * @date  2025/02/14 11:33
     */
    @Operation(summary = "获取清除记录分页")
    @SaCheckPermission("/biz/disablerecord/page")
    @GetMapping("/biz/disablerecord/page")
    public CommonResult<Page<BizDisableRecord>> page(BizDisableRecordPageParam bizDisableRecordPageParam) {
        return CommonResult.data(bizDisableRecordService.page(bizDisableRecordPageParam));
    }

    /**
     * 添加清除记录
     *
     * @author fanzherong
     * @date  2025/02/14 11:33
     */
    @Operation(summary = "添加清除记录")
    @CommonLog("添加清除记录")
    @SaCheckPermission("/biz/disablerecord/add")
    @PostMapping("/biz/disablerecord/add")
    public CommonResult<String> add(@RequestBody @Valid BizDisableRecordAddParam bizDisableRecordAddParam) {
        bizDisableRecordService.add(bizDisableRecordAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑清除记录
     *
     * @author fanzherong
     * @date  2025/02/14 11:33
     */
    @Operation(summary = "编辑清除记录")
    @CommonLog("编辑清除记录")
    @SaCheckPermission("/biz/disablerecord/edit")
    @PostMapping("/biz/disablerecord/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizDisableRecordEditParam bizDisableRecordEditParam) {
        bizDisableRecordService.edit(bizDisableRecordEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除清除记录
     *
     * @author fanzherong
     * @date  2025/02/14 11:33
     */
    @Operation(summary = "删除清除记录")
    @CommonLog("删除清除记录")
    @SaCheckPermission("/biz/disablerecord/delete")
    @PostMapping("/biz/disablerecord/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizDisableRecordIdParam> bizDisableRecordIdParamList) {
        bizDisableRecordService.delete(bizDisableRecordIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取清除记录详情
     *
     * @author fanzherong
     * @date  2025/02/14 11:33
     */
    @Operation(summary = "获取清除记录详情")
    @SaCheckPermission("/biz/disablerecord/detail")
    @GetMapping("/biz/disablerecord/detail")
    public CommonResult<BizDisableRecord> detail(@Valid BizDisableRecordIdParam bizDisableRecordIdParam) {
        return CommonResult.data(bizDisableRecordService.detail(bizDisableRecordIdParam));
    }
}
